@@ -288,19 +288,26 @@ See also `setq'. Returns the value of the last FORM."
 ;; XXX it would be nice to do the same for setq.. might stress the
 ;; XXX interpreter somewhat..? :-(
 
-(defmacro define-special-variable (var #!optional value doc)
-  "define-special-variable VARIABLE [VALUE [DOC]]
+(defmacro defvar-setq (var #!optional value doc)
+  "defvar-setq VARIABLE [VALUE [DOC]]
 
-Declares the symbol VARIABLE as a special variable, then
-unconditionally sets its value to VALUE (or false if VALUE isn't
-defined). If DOC is given it will be installed as the documentation
-string associated with VARIABLE."
+Declares the symbol VARIABLE as a special variable, and sets its value
+to VALUE (or false if VALUE isn't defined). If DOC is given it will be
+installed as the documentation string associated with VARIABLE.
+
+It's the same as the previous `define-special-variable'."
 
   (list 'progn
 	(list 'defvar var nil doc)
 	(list 'setq var value)))
 
-(export-bindings '(setq-default define-special-variable))
+(defmacro define-special-variable (var #!optional value doc)
+  "Use defvar-setq instead."
+  (list 'progn
+	(list 'defvar var nil doc)
+	(list 'setq var value)))
+
+(export-bindings '(setq-default defvar-setq define-special-variable))
 
 
 ;; Misc syntax
@@ -431,13 +438,13 @@ of the possible declaration types.")
 	 (raise-exception data)
        (let ((type (nth 1 data)))
 	 (let loop ((rest handlers))
-	   (if (null rest)
-	       (raise-exception data)
-	     (let ((h-type (caar rest)))
-	       (if (or (and (listp h-type) (memq type h-type))
-		       (eq h-type 'error) (eq h-type type))
-		   ((cdar rest) (cdr data))
-		 (loop (cdr rest)))))))))))
+	      (if (null rest)
+		  (raise-exception data)
+		(let ((h-type (caar rest)))
+		  (if (or (and (listp h-type) (memq type h-type))
+			  (eq h-type 'error) (eq h-type type))
+		      ((cdar rest) (cdr data))
+		    (loop (cdr rest)))))))))))
 
 (defmacro catch (tag . body)
   "Evaluate BODY in an implicit progn; non-local exits are allowed with
@@ -474,7 +481,8 @@ If VAR is true it's a symbol whose values is bound to `(ERROR-SYMBOL .
 DATA)' while the handler is evaluated (these are the arguments given to
 `signal' when the error was raised)."
   (list* 'call-with-error-handlers
-	 (list 'lambda '() form)
+	 (list 'lambda '()
+	       (list 'let '((%in-condition-case t)) form))
 	 (mapcar (lambda (h)
 		   (list 'cons (list 'quote (car h))
 			 (list* 'lambda (and (symbolp var)
