@@ -107,7 +107,19 @@
 			       (memq (car out) top-level-compiled))))))
       (case (car form)
 	((defun)
-	 (remember-function (nth 1 form) (nth 2 form) (nthcdr 3 form)))
+	 (remember-function (nth 1 form) (nth 2 form) (nthcdr 3 form))
+	 (let* ((body (cdddr form))
+		(doc (car body))
+		prop-name)
+	   (when (and (not *compiler-write-docs*)
+		      (stringp doc))
+	     (setq prop-name
+		   (intern
+		    (concat "documentation#"
+			    (symbol-name (fluid current-module)))))
+	     (setq form
+		   `(progn (put ',(cadr form) ',prop-name ,doc)
+			   ,form)))))
 
 	((defmacro)
 	 (remember-function (nth 1 form) (nth 2 form))
@@ -747,7 +759,7 @@
 	 (start-label (make-label))
 	 (end-label (make-label))
 	 (handlers (nthcdr 3 form)))
-    (let-fluids ((lexically-pure nil))
+      (let-fluids ((lexically-pure nil))
 
       ;;		jmp start
       ;; cleanup:
@@ -828,7 +840,9 @@
       (emit-insn '(binderr))
       (increment-b-stack)
       (decrement-stack)
-      (compile-form-1 (nth 2 form))
+
+      (compile-form-1
+       (list 'let '((%in-condition-case t)) (nth 2 form)))
 
       ;; end:
       ;;		unbind			;unbind error handler or VAR
